"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CLIAction = void 0;
const error_codes_1 = require("./error_codes");
/**
 * A wrapper for the command action callback
 */
class CLIAction {
    /**
     * Create an instance of CLIAction
     * @param {AsyncActionCallback} actionCallback the handler method of the action
     */
    constructor(actionCallback) {
        this.actionCallback = actionCallback;
        this._parsedParameters = {};
    }
    get promiseInstance() {
        if (!this._promiseInstance) {
            throw new Error(`There's no instance of a promise before calling it`);
        }
        return this._promiseInstance;
    }
    get parsedParameters() {
        if (!this._parsedParameters) {
            throw new Error(`There's no instance of a promise before calling it`);
        }
        return this._parsedParameters;
    }
    /**
     * A static member refering to the executed action
     * @name runningAction
     * @type {CLIAction}
     * @throws - When read before any action has run
     */
    static get runningAction() {
        if (!this._runningAction) {
            throw new Error(`No action has been called yet`);
        }
        return this._runningAction;
    }
    /**
     * Triggers the callback of the action and returns its promise
     * @name trigger
     * @param {ParsedParameters} params - A key/value dict representing the parsed argv
     * @returns {Promise<ActionReturnType>} - The promise of the callback
     */
    trigger(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._promiseInstance = this.actionCallback(params);
            CLIAction._runningAction = this;
            this._parsedParameters = params;
            return this.promiseInstance
                .then((exitCode) => {
                this.resolveAwaiter();
                return exitCode;
            })
                .catch((err) => {
                console.log(`${err.name}: ${err.message}`);
                this.rejectAwaiter(err);
                return error_codes_1.ERROR_EXIT_CODE;
            });
        });
    }
    /**
     * A hook promise to the action callback
     * @returns {Promise<ParsedParameters} - Resolves into the parsed parameters; rejects if the callback throws, or if another action has run
     */
    actionAwaiter() {
        if (this._promiseInstance) {
            // the promise was already called, resolve when the action is finished
            return this.promiseInstance.then(() => this.parsedParameters);
        }
        else {
            // queue the awaiter for when the promise is called
            const awaiter = new Promise((resolve, reject) => {
                this.awaiterDone = function (value, error) {
                    if (value) {
                        resolve(value);
                    }
                    else {
                        reject(error);
                    }
                };
            });
            return awaiter;
        }
    }
    resolveAwaiter() {
        if (this.awaiterDone) {
            const awaiter = this.awaiterDone;
            awaiter(this.parsedParameters);
        }
    }
    rejectAwaiter(err) {
        if (this.awaiterDone) {
            const awaiter = this.awaiterDone;
            awaiter(undefined, err);
        }
    }
    /**
     * Rejects all the awaiters of this specific action with the given err
     * @param {Error} err - The error thrown while parsing the argv
     * @returns {void}
     */
    setParsingError(err) {
        this.rejectAwaiter(err);
    }
    /**
     * Rejects all the awaiters of this specific action because another action has run
     * @returns {void}
     */
    wasNotTriggered() {
        this.rejectAwaiter(new Error(`Action didn't run`));
    }
}
exports.CLIAction = CLIAction;
