"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CLICommand = exports.assertForbidden = exports.assertRequired = exports.assertConjunctionParameters = exports.programApi = void 0;
/* eslint-disable @typescript-eslint/explicit-module-boundary-types */
const commander_1 = require("commander");
const error_codes_1 = require("./error_codes");
const parameter_1 = require("./parameter");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const { version: CLI_APP_VERSION } = require('../../package.json');
const programAsUnknown = commander_1.program;
exports.programApi = programAsUnknown;
exports.programApi.name('ardrive');
exports.programApi.version(CLI_APP_VERSION);
exports.programApi.addHelpCommand(true);
exports.programApi.usage('[command] [command-specific options]');
// Override the commander's default exit (process.exit()) to avoid abruptly interrupting the script execution
exports.programApi.exitOverride();
/**
 * @name setCommanderCommand
 * @param {CommandDescriptor} commandDescriptor the description of the command to be set
 * @param {CliApiObject} program the instance of the commander class
 * This function is the responsible to tell the third party library to declare a command
 */
function setCommanderCommand(commandDescriptor, program) {
    let command = program.command(commandDescriptor.name);
    const parameters = commandDescriptor.parameters.map((param) => new parameter_1.Parameter(param));
    parameters.forEach((parameter) => {
        const aliasesAsString = parameter.aliases.join(' ');
        const paramTypeString = (function () {
            if (parameter.type === 'array') {
                return ` <${parameter.name}...>`;
            }
            else if (parameter.type === 'boolean') {
                return '';
            }
            return ` <${parameter.name}>`;
        })();
        const optionArguments = [
            `${aliasesAsString}${paramTypeString}`,
            parameter.description,
            parameter.default
        ];
        if (parameter.required) {
            command.requiredOption(...optionArguments);
        }
        else {
            command.option(...optionArguments);
        }
    });
    command = command.action((options) => {
        try {
            assertConjunctionParameters(commandDescriptor, options);
            commandDescriptor.action.trigger(options).then((exitCode) => {
                exitProgram(exitCode || 0);
            });
        }
        catch (e) {
            console.error(`Error: ${e.message}`);
            exitProgram(error_codes_1.ERROR_EXIT_CODE);
        }
    });
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function assertConjunctionParameters(commandDescriptor, options) {
    const parameters = commandDescriptor.parameters;
    parameters.forEach((param) => {
        const parameter = new parameter_1.Parameter(param);
        const parameterName = parameter.name;
        const parameterValue = options[parameterName];
        if (parameterValue) {
            assertRequired(parameter, options);
            assertForbidden(parameter, options);
        }
    });
}
exports.assertConjunctionParameters = assertConjunctionParameters;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function assertRequired(parameter, options) {
    const parameterName = parameter.name;
    const required = parameter.requiredParametersInConjunction;
    required.forEach((requiredParameterName) => {
        const requiredParameterValue = options[requiredParameterName];
        if (!requiredParameterValue) {
            throw new Error(`Parameter ${parameterName} requires ${requiredParameterName} but it wasn't provided`);
        }
    });
}
exports.assertRequired = assertRequired;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function assertForbidden(parameter, options) {
    const parameterName = parameter.name;
    const forbidden = parameter.forbiddenParametersInConjunction;
    forbidden.forEach((forbiddenParameterName) => {
        const forbiddenParameterValue = options[forbiddenParameterName];
        if (forbiddenParameterValue) {
            throw new Error(`Parameter ${parameterName} cannot be used in conjunction with ${forbiddenParameterName}`);
        }
    });
}
exports.assertForbidden = assertForbidden;
class CLICommand {
    /**
     * @param {CommandDescriptor} commandDescription an immutable representation of a command
     * @param {string[]} argv a custom argv for testing purposes
     */
    constructor(commandDescription, program = exports.programApi) {
        this.commandDescription = commandDescription;
        setCommanderCommand(this.commandDescription, program);
        CLICommand.allCommandInstances.push(this);
    }
    get action() {
        return this.commandDescription.action.actionAwaiter();
    }
    static parse(program = exports.programApi, argv = process.argv) {
        program.parse(argv);
        this.rejectNonTriggeredAwaiters();
    }
    static rejectNonTriggeredAwaiters() {
        // reject all action awaiters that haven't run
        const theOtherCommandActions = CLICommand.getAllCommandDescriptors().map((descriptor) => descriptor.action);
        theOtherCommandActions.forEach((action) => action.wasNotTriggered());
    }
    /**
     * @returns {CommandDescriptor[]} all declared command descriptors
     */
    static getAllCommandDescriptors() {
        return this.allCommandInstances.map((cmd) => cmd.commandDescription);
    }
}
exports.CLICommand = CLICommand;
CLICommand.allCommandInstances = [];
function exitProgram(exitCode) {
    process.exitCode = exitCode;
}
