import { ParameterName } from './parameter';
import { DriveID, DriveKey, WalletDAO, Wallet, ArweaveAddress, FeeMultiple, PrivateKeyData, FileNameConflictResolution, CustomMetaData } from 'ardrive-core-js';
declare type ParameterOptions = any;
interface GetDriveKeyParams {
    driveId: DriveID;
    drivePassword?: string;
    useCache?: boolean;
}
interface GetCustomMetaDataWithIpfsCidParameter {
    localFilePath: FilePath;
}
declare type FilePath = string;
/**
 * @type {ParametersHelper}
 * A class that assists with handling Commander options during common ArDrive CLI workflows
 */
export declare class ParametersHelper {
    private readonly options;
    private readonly walletDao;
    private static readonly driveKeyCache;
    /**
     * @returns {ParametersHelper}
     * @param {any} options The object containing the parameterName: value mapping
     * An immutable instance of ParametersHelper holding the parsed values of the parameters
     */
    constructor(options: ParameterOptions, walletDao?: WalletDAO);
    /**
     * @returns {Promise<boolean>}
     * Returns true when a drive password or drive key is provided
     */
    getIsPrivate(): Promise<boolean>;
    /**
     * @returns {Promise<Wallet>}
     * Will return a wallet instance created from the seed phrase or the walletFile.
     * Throws an error if a wallet can't be created.
     */
    getRequiredWallet(): Promise<Wallet>;
    getOptionalWallet(): Promise<Wallet | null>;
    getWalletAddress(): Promise<ArweaveAddress>;
    getOptionalBoostSetting(): FeeMultiple | undefined;
    getPrivateKeyData(): Promise<PrivateKeyData>;
    getDriveKey({ driveId, drivePassword, useCache }: GetDriveKeyParams): Promise<DriveKey>;
    getDrivePassword(isForNewDrive?: boolean): Promise<string>;
    getMaxDepth(defaultDepth?: number): Promise<number>;
    getFileNameConflictResolution(): FileNameConflictResolution;
    /**
     * @param {ParameterName} parameterName
     * @returns {string | undefined}
     * Returns the string value for the specific parameter; returns undefined if not set
     */
    getParameterValue<T = string>(parameterName: ParameterName, mapFunc?: (input: any) => T): T | undefined;
    /**
     * @param {ParameterName} parameterName
     * @param {(input: any) => T} mapFunc A function that maps the parameter value into a T instance
     * @returns {string | undefined}
     * @throws - When the required parameter value has a falsy value
     * Returns the string value for the specific parameter
     */
    getRequiredParameterValue<T = string>(parameterName: ParameterName, mapFunc?: (input: any) => T): T;
    getCustomMetaDataWithIpfsCid({ localFilePath }: GetCustomMetaDataWithIpfsCidParameter): Promise<CustomMetaData>;
    getCustomMetaData(): CustomMetaData | undefined;
    private readMetaDataFromPath;
    private mapMetaDataArrayToCustomMetaDataShape;
    private assertEvenTags;
    private parseMetaDataJson;
    /**
     * Gathers a valid gateway URL from user provided gateway parameter,
     * an environment variable, or returns the default arweave gateway
     *
     * @throws on user provided gateways that are incompatible with URL class constructor
     * @throws when hostName cannot be derived from a user provided gateway
     */
    getGateway(): URL;
    isDryRun(): boolean;
}
export {};
